<?php

namespace App\Http\Controllers;

use App\Models\Fridge;
use App\Models\Stock;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\Auth;

class FridgeController extends Controller
//handles fridge actions
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $fridges = Fridge::where('user_id', auth()->user()->id)->get();
        return view('fridges.index', compact('fridges'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('fridges.create', compact("fridge"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        // ToDo: validation (method for MAC-ADDRESSES ??)
        $request->validate([
            'name' => 'required|string',
            'mac' => 'required|unique:fridges,device|size:17',
            'location' => 'required|string'
        ]);

        $fridge = new Fridge();
        $fridge->user_id = auth()->user()->id;
        $fridge->device = $request->mac;
        $fridge->name = $request->name;
        $fridge->location = $request->location;
        $fridge->save();

        return redirect()->action([FridgeController::class, 'index']);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Fridge  $fridge
     * @return \Illuminate\Http\Response
     */
    public function show(Fridge $fridge)
    {
        //
        return view('fridges.show', compact('fridge'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Fridge  $fridge
     * @return \Illuminate\Http\Response
     */
    public function edit(Fridge $fridge)
    {
        return view('fridges.edit', compact('fridge'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Fridge  $fridge
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Fridge $fridge)
    {
        $request->validate([
            'name' => 'required|string',
            'location' => 'required|string'
        ]);
        $updateFridge = Fridge::find($fridge->id);
        $updateFridge->name = $request->name;
        $updateFridge->location = $request->location;
        $updateFridge->save();

        return redirect()->action([FridgeController::class, 'index']);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Fridge  $fridge
     * @return \Illuminate\Http\Response
     */
    public function destroy(Fridge $fridge)
    {
        $fridge->delete();
        return redirect()->action([FridgeController::class, 'index']);

    }

}
