<?php

namespace App\Http\Controllers;

use App\Models\Fridge;
use App\Models\User;
use App\Models\Product;
use App\Models\Stock;
use Illuminate\Http\Request;
use App\Models\Scan;
use Illuminate\Support\Arr;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;

class ScanController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        // ToDo: validatie uitvoeren indien scan gemaakt is en tekst aanpassen in blade
        $fridges = Fridge::where('user_id', auth()->user()->id)->get();
        $list = $fridges->pluck('id');
        $minimum = 0;
        return view('scans.index', compact('fridges', 'list', 'minimum'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request)
    {
            $request->validate([
                'barcode' => 'required|min:3|max:13',
                'fridge' => 'required'
            ]);

        $fridge = Fridge::where('id', $request->fridge)->first();
        if(!$fridge || $fridge->user_id != auth()->user()->id) {
            abort(401);
        }

        $product = Product::where('barcode', $request->barcode)->first();
        if(!$product) {
            //if product not found in db product create db product
            $api = app('App\Http\Controllers\apiController')->getProductData($request->barcode);
            if($api->status == "not_found"){
                return "404";
            }
            $product = Product::create([
                'barcode' => $api->barcode,
                'name' => $api->name,
                'size' => $api->size,
                'brand' => $api->brand,
                'store' => $api->store,
                'image' => $api->image,
            ])->save();
            $product = Product::where('barcode', $api->barcode)->first();
        }
        return view('scans.create', compact('product', 'fridge'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'barcode' => 'required|min:3|max:13',
            'fridge_id' => 'required|exists:fridges,id',
            'product_id' => 'required|exists:products,id',
            'amount' => 'required|integer|min:1',
            'expire' => 'nullable|date'
        ]);

        $fridge = Fridge::where('id', $request->fridge_id)->first();
        $product = Product::where('id', $request->product_id)->first();

        if(!$product || !$fridge || $fridge->user_id != auth()->user()->id) {
            abort(401);
        }

        if($request->expire){
            $expire = $request->expire;
        }else{
            $expire = null;
        }

        $amount = $request->amount;

        $stock = Stock::where('expire', $expire)->where('product_id', $product->id)->where('fridge_id', $fridge->id)->first();
        if(!isset($stock) && $amount < 0){
            return "425";
        }

        //create db scan
        $scan = new Scan();
        $scan->fridge_id = $fridge->id;
        $scan->product_id = $product->id;
        $scan->amount = $amount;
        $scan->expire = $expire;
        $scan->save();

        //updateOrCreate db stock
        $stock = Stock::updateOrInsert(
            ['expire' => $expire, 'fridge_id' => $fridge->id, 'product_id' => $product->id],
            ['amount' => DB::raw('amount + '.$amount), 'created_at' => Carbon::now(), 'updated_at' => Carbon::now()]
        );

        return redirect()->action([ScanController::class, 'index']);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Scan  $scan
     * @return \Illuminate\Http\Response
     */
    public function show(Scan $scan)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Scan  $scan
     * @return \Illuminate\Http\Response
     */
    public function edit(Scan $scan)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Scan  $scan
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Scan $scan)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Scan  $scan
     * @return \Illuminate\Http\Response
     */
    public function destroy(Scan $scan)
    {
        //
    }
}
