<?php

namespace App\Http\Controllers;

use App\Models\Stock;
use App\Models\User;
use App\Http\Controllers\ProductController;
use App\Http\Controllers\ScanController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Http;
use App\Models\Scan;

class StockController extends Controller
{

    function addStock(Array $reg){
        $newStock = new Stock();
        $newStock->id = NULL;
        $newStock->user_id = auth()->user()->id;
        $newStock->product_id = $reg['product_id'];
        $newStock->amount = $reg['amount'];
        $newStock->expire = $reg['date'];
        $newStock->save();
        return true;
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        // TEMPORARY
        return view('stocks.index');

        $stocks=Stock::where('user_id', auth()->user()->id)->orderBy('product_id')->orderBy('expire', 'asc')->paginate(4);

        return view('stocks.index', compact('stocks'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        // ToDo: notifications on success
        // ToDo: error text on not found
        $validator = request()->validate([
            'barcode' => 'required'
        ]);

        $product = app(ProductController::class)->getProductInfo(request()->barcode);

        switch($product['status']){
            case "ok": //product found through api
                return view('stocks.create', compact('product'));
                break;
            case "exists": //product found in db
                return view('stocks.create', compact('product'));
                break;
            case "not_found": //not found in api nor db
                return redirect('/stocks')->withErrors($validator)->withInput();
                break;
        }
        return redirect('/stocks');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $reg)
    {
        // ToDo: fix the request validate for date
        if($reg->datum ==NULL){
            return redirect('/stocks');
        }
        request()->validate([
            'amount' => 'required',
            'barcode' => 'required',
        ]);

        $barcode = $reg->barcode;
        if(auth()->user()){

            $product = app(ProductController::class)->getProductInfo($reg->barcode);//request data about the product we want to store
            $product += [
                'amount' => $reg->amount,
                'date' => $reg->datum,
            ];
            if($product['status'] == "ok"){//product doesnt exit yet so create a new one
                $newId = app(ProductController::class)->newProduct($product);//new id and create product
                $product += ['product_id' => $newId];
            }
            else if($product['status'] == "not_found"){//product not found for given barcode
                abort(404);
            }
            app(ScanController::class)->addScan($product);//add a scan

            //product already exists in product db
            //do a check here if the user already owns this product with matching date
            //if(user already owns similar product)
            //..
            //return redirect...
            //else
            $this->addStock($product);
            return redirect()->action([StockController::class, 'index']);
            //return redirect('/stocks');

        }
        else{
            abort(401);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Stock $stock)
    {
        //

        if(auth()->user()->id <> $stock->fridge->user_id) {
            abort(401);
        }

        //$stocks = Stock::find($id);
        return view('stocks.show', compact('stock'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Stock $stock)
    {
        //
        //return $stock->product;
        if(auth()->user()->id <> $stock->fridge->user_id) {
            abort(401);
        }


        return view('stocks.edit', compact('stock'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Stock $stock)
    {
        /*
        request()->validate([
            'date' => 'required',
            'amount' => 'required',
        ]);
        */

        if(auth()->user()->id <> $stock->fridge->user_id) {
            abort(401);
        }
        $amount = 0;

        if(request()->add) {
            $stock->amount = $stock->amount + 1;
            //dd($stocks->amount);
            $stock->save();
            $amount = 1;
        }

        if(request()->subtract) {
            $stock->amount = $stock->amount - 1;
            //dd($stocks->amount);
            $stock->save();
            $amount = -1;
        }
        //create db scan
        $scan = new Scan();
        $scan->fridge_id = $stock->fridge->id;
        $scan->product_id = $stock->product->id;
        $scan->amount = $amount;
        $scan->expire = $stock->expire;
        $scan->save();

        $isZero = Stock::where('amount', '<=', 0)->delete();
        if($isZero){
            return redirect('/dashboard');
        }

        Stock::where('amount', '<=', 0)->delete();
        return redirect()->action([StockController::class, 'edit'], $stock);
        //return redirect('/stocks');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Stock $stock)
    {
        $stock->delete();
        return redirect('/stocks');
    }
}
