<?php

namespace App\Http\Controllers;

use DateTime;
use Illuminate\Http\Request;
use Symfony\Component\Console\Input\StringInput;
use App\Models\Product;
use App\Models\Stock;
use Illuminate\Support\Arr;
use App\Models\Fridge;
use App\Models\Scan;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;


//APICONTROLLER visits the api of different supermarkets and looks up product data by barcode + code for own api for arduino
class apiController extends Controller
{
    //general function that tries to get any response from multiple shops
    function getProductData(String $barcode){
        if($barcode == NULL){
            abort(502);
        }
        $plusReturn = $this->plusAPI($barcode);
        if($plusReturn['status'] == "ok"){ //try plus api
            return (object)$plusReturn;
        }
        $ahReturn = $this->ahAPI($barcode);
        if($ahReturn['status'] == "ok"){ //try ah api
            return (object)$ahReturn;
        }
        //try the coop
        $coopReturn = $this->jumbOffCoopAPI($barcode, "coop");
        if($coopReturn['status'] == "ok"){
            return (object)$coopReturn;
        }
        //try the jumbo
        $jumboReturn = $this->jumbOffCoopAPI($barcode, "jumbo");
        if($jumboReturn['status'] == "ok"){
            return (object)$jumboReturn;
        }
        //try the off api
        $offReturn = $this->jumbOffCoopAPI($barcode, "off");
        if($offReturn['status'] == "ok"){
            return (object)$offReturn;
        }
        return (object)$offReturn;

    }

    //ah api url's taken from: https://github.com/bartmachielsen/SupermarktConnector
    function ahAPI(String $barcode){

        //initialise curl to get key
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => "https://ms.ah.nl/create-anonymous-member-token?client=appie-anonymous",
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_POSTFIELDS => "{\n\"api_token\": \"[ADD YOUR TOKEN]\"\n}",
          CURLOPT_HTTPHEADER => array(
            "Accept: */*",
            "Cache-Control: no-cache",
            "Content-Type: application/json",
            "Host: ms.ah.nl",
            "User-Agent: android/6.29.3 Model/phone Android/7.0-API24",
          ),
        ));
        //process and close the curl
        $response = curl_exec($curl);
        curl_close($curl);
        $key = json_decode($response);

        //filter for error in key request
        if($key == NULL){
            $compiled_data = [
                "status" => "error",
                "message"=> "Error in retrieving api key for ah",
                "store" => "ah",
            ];
            return $compiled_data;
        }

        //initialise new curl to request file data
        $ah_curl = curl_init();
        curl_setopt_array($ah_curl, array(
            CURLOPT_URL => "https://ms.ah.nl/mobile-services/product/search/v1/gtin/$barcode",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_POSTFIELDS => "{\n\"api_token\": \"[ADD YOUR TOKEN]\"\n}",
            CURLOPT_HTTPHEADER => array(
              "Accept: */*",
              "Cache-Control: no-cache",
              "Content-Type: application/json",
              "Host: ms.ah.nl",
              "User-Agent: android/6.29.3 Model/phone Android/7.0-API24",
              "Authorization: Bearer $key->access_token"
            ),
          ));
          //fetch data and close curl
          $ah_respons = curl_exec($ah_curl);
          curl_close($ah_curl);

          //check if there was returned a valid response
          if (!empty($ah_respons)){
              //processing the data
            $ah_decoded_response = json_decode($ah_respons);
              $compiled_data = [
                  "status" => "ok",
                  "name" => "$ah_decoded_response->title",
                  "size" => "$ah_decoded_response->salesUnitSize",
                  "brand" => "$ah_decoded_response->brand",
                  "barcode" => $barcode,
                  "store" => "ah",
                  "image" => $ah_decoded_response->images[2]->url,
              ];
            return $compiled_data;
         }
         //return error if there was nothing found
         else{
            $compiled_data = [
                "status" => "not_found",
                "message"=> "no product found for $barcode",
                "store" => "ah",
            ];
            return $compiled_data;
         }
    }

    //jumbo api url taken from: https://github.com/bartmachielsen/SupermarktConnector
    //coop api found by Menke Veerman
    function jumbOffCoopAPI(String $barcode, String $apiType){
        if($apiType == "jumbo"){ //jumbo request
            $url = "https://mobileapi.jumbo.com/v9/search?q=$barcode";
        }
        else if($apiType == "coop"){ //coop request
            $url = "http://www.coop.nl/INTERSHOP/rest/WFS/COOP-COOPBase-Site/-/deprecated-products?searchTerm=$barcode";
        }
        else{ //off request
            $url = "https://world.openfoodfacts.org/api/v0/product/$barcode.json";
        }

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => "$url",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => array(
              "Accept: */*",
              "Cache-Control: no-cache",
              "Connection: keep-alive",
              "Content-Type: application/json",
            ),
          ));

          $response = curl_exec($curl);
          $response_decoded = json_decode($response);

        if($apiType == "jumbo"){ //jumbo request process data
            if(!$response_decoded->products->data){ //bad result
                $compiled_data = [
                    "status" => "not_found",
                    "message"=> "no product found for $barcode",
                    "store" => "jumbo",
                ];
            }
            else{ //was able to recevie data from jumbo api
                $compiled_data = [
                    "status" => "ok",
                    "name" => $response_decoded->products->data[0]->title,
                    "size" => $response_decoded->products->data[0]->quantity,
                    "brand" => $response_decoded->products->data[0]->title,
                    "barcode" => $barcode,
                    "store" => "jumbo",
                    "image" => $response_decoded->products->data[0]->imageInfo->primaryView[0]->url,
                ];
            }
            return $compiled_data;
        }
        else if($apiType == "coop"){ //jumbo request process data
            if(!$response_decoded->elements){ //bad result
                $compiled_data = [
                    "status" => "not_found",
                    "message"=> "no product found for $barcode",
                    "store" => "jumbo",
                ];
            }
            else{ //was able to recevie data from jumbo api
                $compiled_data = [
                    "status" => "ok",
                    "name" => $response_decoded->elements[0]->title,
                    "size" => $response_decoded->elements[0]->inhoud,
                    "brand" => $response_decoded->elements[0]->title,
                    "barcode" => $barcode,
                    "store" => "coop",
                    "image" => $response_decoded->elements[0]->images->M,
                ];
            }
            return $compiled_data;
        }
        else{ //off request process data
            if($response_decoded->status_verbose == "product not found"){ //bad result
                $compiled_data = [
                    "status" => "not_found",
                    "message"=> "no product found for $barcode",
                    "store" => "off",
                ];
            }
            else{ //was able to recevie data from off api
                $compiled_data = [
                    "status" => "ok",
                    "name" => $response_decoded->product->product_name,
                    "size" => $response_decoded->product->quantity,
                    "brand" => $response_decoded->product->brands,
                    "barcode" => $barcode,
                    "store" => "off",
                    "image" => $response_decoded->product->image_url,
                ];
            }
            return $compiled_data;
        }

    }

    //plus api found by Menke Veerman
    function plusAPI(String $barcode){

        //initialise curl to get the sku-id linked to the barcode
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => "https://pls-sprmrkt-mw.prd.vdc1.plus.nl/api/v2/proxy/eancode/$barcode",
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_POSTFIELDS => "{\n\"api_token\": \"[ADD YOUR TOKEN]\"\n}",
          CURLOPT_HTTPHEADER => array(
            "Accept: */*",
            "Cache-Control: no-cache",
            "Content-Type: application/json",
            "Host: pls-sprmrkt-mw.prd.vdc1.plus.nl",
            "User-Agent: android/6.29.3 Model/phone Android/7.0-API24",
          ),
        ));
        //process and close the curl
        $response = curl_exec($curl);
        curl_close($curl);
        $sku = json_decode($response);

        //filter for error in sku request
        if(property_exists($sku, "error")){
            $compiled_data = [
                "status" => "error",
                "message"=> "issue with finding sku id",
                "store" => "plus",
            ];
            return $compiled_data;
        }

        //initialise new curl to request file data
        $plus_curl = curl_init();
        curl_setopt_array($plus_curl, array(
            CURLOPT_URL => "https://pls-sprmrkt-mw.prd.vdc1.plus.nl/api/v3/product/$sku->sku",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_POSTFIELDS => "{\n\"api_token\": \"[ADD YOUR TOKEN]\"\n}",
            CURLOPT_HTTPHEADER => array(
              "Accept: */*",
              "Cache-Control: no-cache",
              "Content-Type: application/json",
              "Host: pls-sprmrkt-mw.prd.vdc1.plus.nl",
              "User-Agent: android/6.29.3 Model/phone Android/7.0-API24",
            ),
          ));
          //fetch data and close curl
          $plus_respons = curl_exec($plus_curl);
          curl_close($plus_curl);

          //decode fetched data
          $plus_decoded_response = json_decode($plus_respons);

          //check if there was returned a valid response
          if (!property_exists($sku, "error")){
              //processing the data
              $compiled_data = [
                  "status" => "ok",
                  "name" => "$plus_decoded_response->name",
                  "size" => "$plus_decoded_response->ratioBasePackingUnit $plus_decoded_response->baseUnit",
                  "brand" => "$plus_decoded_response->merk",
                  "barcode" => $barcode,
                  "store" => "plus",
                  "image" => "https://www.plus.nl$plus_decoded_response->image_L",
              ];
            return $compiled_data;
         }
         //return error if there was nothing found
         else{
            $compiled_data = [
                "status" => "not_found",
                "message"=> "no product found for $barcode",
                "store" => "plus",
            ];
            return $compiled_data;
         }
    }

    //own api for the arduino to talk with
    public function scanner(Request $request) {
        $response = null;
        //return $request;
        //$response = response('404 - Product not found', 404);
        //$response = response('500 - Server problem', 500);
        //$response = response('200 - Product $product added', 200);

        //validate $request;
        $user_id = $request->user;
        $mac = $request->fridge;
        $barcode = $request->barcode;
        $amount = $request->amount;
        $expire = null;

        //check three required
        if(!isset($user_id) || !isset($mac) || !isset($barcode) || strlen($mac) != 17){
            return "417";

        }

        //check date
        if(isset($request->expire) && strlen($request->expire) == 8){
            $expire = Carbon::parse($request->expire)->format('Y-m-d');
        }
        else if(isset($request->expire)){
            echo "422";
            return;
        }


        //validation fridge / user combination
        $fridge = Fridge::where('device', '=', $mac)->first();
        if(!$fridge){
            echo "418";
            return "";
        } else if( $fridge->user_id != $user_id ){
            return "419";
        }

        //search barcode in api/database
        $product = Product::where('barcode', $request->barcode)->first();
        if(!$product) {
            //if product not found in db product create db product
            $api = app('App\Http\Controllers\apiController')->getProductData($request->barcode);
            if($api->status == "not_found"){
                echo "404";
                return;
            }
            $product = Product::create([
                'barcode' => $api->barcode,
                'name' => $api->name,
                'size' => $api->size,
                'brand' => $api->brand,
                'store' => $api->store,
                'image' => $api->image,
            ])->save();
            $product = Product::where('barcode', $api->barcode)->first();
        }
        if(!isset($amount)){
            $amount = 1;
        }else if($amount == 0){
            echo "423";
            return;
        }

        $stock = Stock::where('expire', $expire)->where('product_id', $product->id)->where('fridge_id', $fridge->id)->first();
        if(!isset($stock) && $amount < 0){
            echo "425";
            return;
        }

        //create db scan
        $scan = new Scan();
        $scan->fridge_id = $fridge->id;
        $scan->product_id = $product->id;
        $scan->amount = $amount;
        $scan->expire = $expire;
        $scan->save();

        //updateOrCreate db stock
        $stock = Stock::updateOrInsert(
            ['expire' => $expire, 'fridge_id' => $fridge->id, 'product_id' => $product->id],
            ['amount' => DB::raw('amount + '.$amount), 'created_at' => Carbon::now(), 'updated_at' => Carbon::now()]
        );
        Stock::where('amount', '<=', 0)->delete();
        //return response
        if($amount < 0){
            echo "200";
            return;
        }else{
            echo $product->name; //added succesfully
            return;
        }
    }
}
