@extends('pages.layout.base')

@section('content')
<main class="flex-1 relative z-0 overflow-y-auto focus:outline-none tabindex="0">

    <div class="flex mt-8 mx-8">
        <div class="text-3xl leading-6 font-bold text-gray-500">
            My fridge: {{ $fridge->name }} -
        </div>
        <div class="ml-2 text-xl text-gray-400 font bold ">
            Located at the {{ $fridge->location }}
        </div>
    </div>
    <div>
        @php
            $expired = $fridge->stock()->whereDate('expire', '<', Carbon\Carbon::now()->toDateString())->sum('stocks.amount');
            $toExpire = $fridge->stock()->whereDate('expire', '<', Carbon\Carbon::now()->addDays(3)->toDateString())->whereDate('expire', '>=', Carbon\Carbon::now()->addDays(0)->toDateString())->sum('stocks.amount');
            $good = $fridge->stock()->whereDate('expire', '>=', Carbon\Carbon::now()->addDays(3)->toDateString())->sum('stocks.amount')+$fridge->stock()->where('expire', '=', NULL)->sum('stocks.amount');
        @endphp
        <div class="flex items-center ml-8 py-3 text-gray-600 space-x-4 group-hover:bg-blue-50">
            <div class="flex items-center space-x-1">
                <svg class="h-5 w-5 text-green-500" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z" >
                </svg>
                <span>
                    {{ $good }}
                </span>
            </div>
            <div class="flex items-center space-x-1">
                <svg class="h-5 w-5 text-yellow-600"xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 8v4m0 4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z" />
                </svg>
                <span>
                    {{ $toExpire }}
                </span>
            </div>
            <div class="flex items-center space-x-1">
                <svg class="h-5 w-5 text-red-600" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z" />
                </svg>
                <span>
                    {{ $expired }}
                </span>
            </div>
        </div>
    </div>

    <div class="m-8">
        <div class="text-gray-600 mx-auto">
            All products
        </div>
        <ul class="mt-4 grid sm:grid-cols-1 gap-6 md:grid-cols-2 xl:grid-cols-3 2xl:grid-cols-4">
            @foreach($fridge->stock->sortBy('updated_at', SORT_REGULAR, false) as $stock)
                @php
                    $now = Carbon\Carbon::now()->toDateString();
                    $future = Carbon\Carbon::now()->addDays(3)->toDateString();
                    if(isset($stock->expire)){
                        $expire = $stock->expire->toDateString();
                    }
                    else {
                        $expire = Carbon\Carbon::now()->addDays(10)->toDateString();
                    }
                    if($now > $expire){  //expired
                        $color = "text-red-600";
                    }
                    else if($expire >= $future){  //almost expiring
                        $color = "text-green-500";
                    }
                    else{ //fine
                        $color = "text-yellow-600";
                    }
                @endphp
                <a href="{{ route('stocks.edit', $stock->id) }}" class="group relative bg-white rounded-lg shadow-sm cursor-pointer focus:outline-none focus:ring-1 focus:ring-offset-2 focus:ring-indigo-500">
                    <div class="rounded-lg border border-gray-300 bg-white px-6 py-4 hover:border-gray-400 flex justify-between">
                        <div class="flex items-center">
                            <img class="mr-3 border object-cover w-14 h-10 rounded-md" src="{{ $stock->product->image }}" alt="Image">
                            <div class="text-sm">
                                <p class="font-medium text-gray-900 capitalize">
                                    {{ $stock->product->name }}
                                </p>
                                <div class="text-gray-500">
                                    <p class="sm:inline">{{ $stock->product->brand }} - {{ $stock->product->size }}</p>
                                </div>
                            </div>
                        </div>
                        <div class="mt-2 text-sm text-right">
                            <div class="font-medium text-gray-900">
                                # {{ $stock->amount }}
                            </div>

                            <div class="ml-1 {{ $color }} ">
                                @if(isset($stock->expire))
                                    {{ $stock->expire->format('d.m.Y') }}
                                @else
                                    -
                                @endif
                            </div>

                        </div>
                    </div>
                </a>
            @endforeach
        </ul>
    </div>


</main>
@endsection
